/*
 * =====================================================================================
 *
 *       Filename:  fmeas.c
 *
 *    Description:  Frequency measure API
 *
 *        Version:  1.0
 *        Created:  28.03.2014 23:20:47
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lucck(at)boff(dot)pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */
#include <stdbool.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include "fmeas.h"
/* ------------------------------------------------------------------ */ 
#define FIN_DDR DDRD	//! Freqeuency input ddr
#define FIN_PIN _BV(PD5) //! Frequency input pin
/* -------------------------------------------------------------------*/
#define GTIMER_HZ 50UL
#define GTIMER_PRELOAD (((F_CPU)/1024UL)/GTIMER_HZ)
#define GTIMER_COUNT 5
#define FMEAS_HZ_SCALE 40UL //! Define result multiply for receive HZ
/* ------------------------------------------------------------------ */ 
//! Hi timer software cnt
static volatile uint16_t hi_timer;
static volatile fmeas_t g_freq;
/* ------------------------------------------------------------------ */
//! Start or stop gate counting
static inline void gate_ctl( bool en ) 
{
	if( en ) {
		TCNT1 = 0; hi_timer = 0;
		TCCR1B = _BV(CS12) | _BV(CS11) | _BV(CS10);
	} else {
		TCCR1B = 0;
	}
}
/* ------------------------------------------------------------------ */ 
//! Fmeasure initial initialization 
void fmeas_init( void )
{
	FIN_DDR &= ~FIN_PIN;
	// Configure timer 1 for measure
	TCCR1A = 0;
	//!Enable interrupt from timer 0 and over flow 1
	TIMSK |= _BV(TOIE1) | _BV(OCF2);
}
/* ------------------------------------------------------------------ */ 
//! Start the frequency measure
void fmeas_start( void )
{
	OCR2 = GTIMER_PRELOAD - 1UL;
	gate_ctl( true );
	TCCR2 = _BV(WGM21) | _BV(CS22) | _BV(CS21) | _BV(CS20);
}
/* ------------------------------------------------------------------ */ 
//! Stop the frequency measure
void fmeas_stop( void )
{
	gate_ctl( false );
	TCCR2 = 0;
}
/* ------------------------------------------------------------------ */
//! Get current counted freuency
fmeas_t fmeas_get_value(void)
{
 	fmeas_t f1, f2;
	do {
		f1 = g_freq;
		f2 = g_freq;
	} while( f1 != f2 );
	return f1 * FMEAS_HZ_SCALE;
}
/* ------------------------------------------------------------------ */
//! Time 0 overflow vector gate frequency CNT
ISR( TIMER2_COMP_vect ) 
{
	static uint8_t psc;
	if( ++psc == GTIMER_COUNT ) {
		gate_ctl( false );
		g_freq = ((uint32_t)hi_timer << 16) | TCNT1;
		gate_ctl(true);
		psc = 0;
	}
}
/* ------------------------------------------------------------------ */
//! HIGH part time value
ISR( TIMER1_OVF_vect )
{
	++hi_timer;	
}
/* ------------------------------------------------------------------ */
