/*
 * =====================================================================================
 *
 *       Filename:  keypad.c
 *
 *    Description:   Simple keyboard avr implementation
 *
 *        Version:  1.0
 *        Created:  29.03.2014 17:05:39
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lucck(at)boff(dot)pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */
#include "keypad.h"
#include <avr/io.h>
/* ------------------------------------------------------------------ */ 
#define KEY_LEFT _BV(PD2)
#define KEY_RIGHT _BV(PD3)
#define KEY_MASK (KEY_LEFT|KEY_RIGHT)
#define KEY_PORT PORTD
#define KEY_PIN PIND
#define KEY_DDR DDRD
/* ------------------------------------------------------------------ */
static volatile uint8_t key_p = 0;
/* ------------------------------------------------------------------ */ 
//! Initialize the keyboard stuff
void keyb_init(void) 
{
	//! Configure keyb as Input internal pullup
	KEY_DDR &= ~KEY_MASK;
	KEY_PORT |= KEY_MASK;
}
/* ------------------------------------------------------------------ */
//! Read Raw keys
static inline uint8_t raw_keys(void)
{
	return ~( KEY_PIN >> 2 ) & 3;
}
/* ------------------------------------------------------------------ */
//! Get keyboard status
keyb_t keyb_get(void)
{
	uint8_t tmp = key_p;
	key_p = 0;
	return tmp;
}
/* ------------------------------------------------------------------ */
//! Keyboard interrupt handler
void _keyb_time_interrupt(void) 
{
	static uint8_t last_key=0;
	static uint8_t repeat_k=0;
	static uint8_t delay_k=0;
	uint8_t port_key;
	
	if(++repeat_k == 10)
	{
		repeat_k = 0;
		port_key = raw_keys();
		if(port_key)
		{
			if(port_key == last_key)
			{
				if(!delay_k)
				{	
					key_p = port_key | key_rpt;
				}
				else
				{
					--delay_k;
				}
			}
			else
			{
				key_p = last_key = port_key;
				delay_k = 15;
			}
		}
		else
		{
			last_key =0;
		}
	}
}
/* ------------------------------------------------------------------ */
//Return true if key is active
bool kbhit(void)
{
	return raw_keys() != 0;
}
/* ------------------------------------------------------------------ */ 
