/*
 * =====================================================================================
 *
 *       Filename:  poweroff.c
 *
 *    Description:  PWR disable API
 *
 *        Version:  1.0
 *        Created:  30.03.2014 11:06:57
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lucck(at)boff(dot)pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */
#include <avr/io.h>
#include <avr/sleep.h>
#include <util/atomic.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/cpufunc.h>
#include "display.h"
#include "voltmeter.h"
#include "fmeas.h"
#include "poweroff.h"
/* ------------------------------------------------------------------ */
#define PWR_DDR DDRD
#define PWR_PORT PORTD
#define PWR_PIN _BV(PD6)

/* ------------------------------------------------------------------ */
//! Power on the device
void power_on(void) 
{
	//!If it is after power on disable the device
	if( MCUCSR & _BV(PORF) ) {
		MCUCSR = 0;
		power_off();
	}
	ATOMIC_BLOCK( ATOMIC_RESTORESTATE) {
		PWR_DDR |= PWR_PIN;
		PWR_PORT |= PWR_PIN;
	}
	//! Reset status register
	MCUCSR = 0;
}
/* ------------------------------------------------------------------ */
//! Software power down controller and taurus
void power_off(void) 
{
	ATOMIC_BLOCK( ATOMIC_RESTORESTATE ) {
		PWR_PORT &= ~PWR_PIN;
	}
	//! Deinitialize all modules
	displ_deinit();
	vmtr_deinit();
	fmeas_stop();
	//! Configure external interrupt to power on
	ATOMIC_BLOCK( ATOMIC_RESTORESTATE ) {
		MCUCR &= ~( _BV( ISC00 ) | _BV( ISC01 ) );
		GIFR &= ~ _BV(INTF0);
		//External interrupt enable
		GICR |= _BV(INT0);
	}
	set_sleep_mode( SLEEP_MODE_PWR_DOWN );
	wdt_disable();
	sleep_mode();
	ATOMIC_BLOCK( ATOMIC_RESTORESTATE ) {
		GICR &= ~_BV(INT0);
	}
	/** Reset CPU using watchdog event special care must be taken
	 * on never CPUS according to the avr gcc binutuils. On older
	 * cpu like ATMEGA8 any action is not needed
	 */
	wdt_enable( WDTO_15MS );
	for(;;) _NOP();
}
/* ------------------------------------------------------------------ */
//! Dummy ISR0 vector
ISR( INT0_vect ) {
	GICR &= ~_BV(INT0);
}
/* ------------------------------------------------------------------ */

